==> kiss_icp-0.0.9-cp38-cp38-manylinux_2_17_x86_64.manylinux2014_x86_64.whl: kiss_icp/deskew.py <==
# MIT License
#
# Copyright (c) 2022 Ignacio Vizzo, Tiziano Guadagnino, Benedikt Mersch, Cyrill
# Stachniss.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from abc import ABC

from kiss_icp.config import KISSConfig
from kiss_icp.pybind import kiss_icp_pybind


class StubCompensator(ABC):
    def __init__(self):
        pass

    def deskew_scan(self, frame, poses, timestamps):
        return frame


class MotionCompensator:
    def __init__(self, config: KISSConfig):
        self.scan_duration = 1 / config.data.lidar_frequency
        self.mid_pose_timestamp = 0.5  # TODO: Expose this

    # This could be an IMU estimation
    def velocity_estimation(self, poses):
        return kiss_icp_pybind._velocity_estimation(
            start_pose=poses[-2],
            finish_pose=poses[-1],
            scan_duration=self.scan_duration,
        )

    def deskew_scan(self, frame, poses, timestamps):
        if len(poses) < 2:
            return frame

        linear_velocity, angular_velocity = self.velocity_estimation(poses)
        return kiss_icp_pybind._deskew_scan(
            frame=kiss_icp_pybind._Vector3dVector(frame),
            timestamps=self.scan_duration * (timestamps - self.mid_pose_timestamp),
            linear_velocity=linear_velocity,
            angular_velocity=angular_velocity,
        )

==> kiss_icp-1.0.0-cp38-cp38-manylinux_2_17_x86_64.manylinux2014_x86_64.whl: kiss_icp/deskew.py <==
# MIT License
#
# Copyright (c) 2022 Ignacio Vizzo, Tiziano Guadagnino, Benedikt Mersch, Cyrill
# Stachniss.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import numpy as np

from kiss_icp.config import KISSConfig
from kiss_icp.pybind import kiss_icp_pybind


def get_motion_compensator(config: KISSConfig):
    return MotionCompensator() if config.data.deskew else StubCompensator()


class StubCompensator:
    def deskew_scan(self, frame, timestamps, delta):
        return frame


class MotionCompensator:
    def deskew_scan(self, frame, timestamps, delta):
        deskew_frame = kiss_icp_pybind._deskew_scan(
            frame=kiss_icp_pybind._Vector3dVector(frame),
            timestamps=timestamps,
            delta=delta,
        )
        return np.asarray(deskew_frame)
